﻿/*
Autor: Vicente Manuel García Huete - vicman.lobo@gmail.com
Este software está licenciado como Creative Commons Atribución-NoComercial-SinDerivar 4.0 Internacional

Quedan excluídos de la Licencia, y se prohíbe su redistribución, los siguientes recursos con Licencias Privadas:
Directorio: "Assets/EasyMovieTexture" (https://www.assetstore.unity3d.com/en/#!/content/10032)
Directorio: "Assets/Coindrop assets" (https://www.assetstore.unity3d.com/en/#!/content/28039)

Recursos con Licencias Creative Commons Utilizados:
YoutubeExtractor: https://github.com/flagbug/YoutubeExtractor
Youtube Data API v3: https://developers.google.com/youtube/v3/
Wallpaper Tablet: https://www.flickr.com/photos/101039078@N06/9759631834
Sprite de Carga: https://pixabay.com/es/proyector-proyector-cinematogr%C3%A1fico-361784/
Icono Youtube: https://pixabay.com/es/video-jugar-multimedia-youtube-481821/
Icono Imagen: https://commons.wikimedia.org/wiki/File:Image_manquante_2.svg
Iconos Zoom: https://commons.wikimedia.org/wiki/File:Zoom_out_font_awesome.svg
https://commons.wikimedia.org/wiki/File:Zoom_in_font_awesome.svg
Icono Salir: https://pixabay.com/es/registro-fuera-cerrar-sesi%C3%B3n-salida-35496/
Icono Mute: https://commons.wikimedia.org/wiki/File:Speaker_Icon.svg#/media/File:Mute_Icon.svg
Icono Volume Up: https://commons.wikimedia.org/wiki/File:Speaker_Icon.svg
Iconos Reproducción: https://pixabay.com/es/multimedia-botones-jugador-jugar-153230/
Icono Sprite Gaze: https://commons.wikimedia.org/wiki/File:Map_pointer.svg
Modelo Tablet: http://www.modelplusmodel.com/tech/apple-ipad-2.html
Icono Video: https://pixabay.com/es/cuenta-regresiva-cine-pel%C3%ADcula-155439/
Icono Preferencias: https://pixabay.com/es/engranajes-opciones-configuraci%C3%B3n-467261/
Icono Galería: https://pixabay.com/es/portapapeles-pel%C3%ADcula-clap-305778/
Fuente Cinerama: http://www.dafont.com/es/cinerama.font
*/

using UnityEngine;
using System.Collections;

public class Tutorial : MonoBehaviour {

    public Animator lensAdjustCanvas;
    public Animator cursorCanvas;
    public Animator clickCanvas;
    public Animator volumeCanvas;
    public Animator tabletCanvas;
    public Animator interactivityCanvas;
    public Animator closeCanvas;
    public Animator hideSalirDialog;
    public GameObject loadingCanvas;
    public GameObject mCamera;

    public ParticleSystem palomitasParticles;

    public GameObject cupWithLid;
    public GameObject popcornBucket;
    public GameObject ticket;
    public GameObject speaker;
    public GameObject tablet;

    private bool cupRotating = false;
    private bool volup = false;
    private bool play = false;
    private bool quitting = false;


	void Start () {
        lensAdjustCanvas.SetBool("isVisible", true);
        StartCoroutine(LensDistanceTimeOut());
	}

    IEnumerator LensDistanceTimeOut() {
        yield return new WaitForSeconds(7);
        lensAdjustCanvas.SetBool("isVisible", false);
        if (!quitting) {
            cursorCanvas.gameObject.SetActive(true);
            cursorCanvas.SetBool("isVisible", true);
        }
    }

    public void CloseCursorDialog() {
        Debug.Log("CloseCursorDialog: " + cursorCanvas.GetBool("isVisible"));
        if (cursorCanvas.GetBool("isVisible") == true) {
            cursorCanvas.SetBool("isVisible", false);
            cursorCanvas.gameObject.SetActive(false);
            clickCanvas.gameObject.SetActive(true);
            clickCanvas.SetBool("isVisible", true);
        }
    }

    public void CloseClickDialog() {
        palomitasParticles.Play();
        if (clickCanvas.GetBool("isVisible") == true) {
            clickCanvas.SetBool("isVisible", false);
            clickCanvas.gameObject.SetActive(true);
            interactivityCanvas.gameObject.SetActive(true);
            interactivityCanvas.SetBool("isVisible", true);
            tablet.SetActive(true);
            speaker.SetActive(true);
        }
    }

    public void StartRotateCup() {
        if (!cupRotating) StartCoroutine(RotateCup());
    }
    IEnumerator RotateCup() {
        cupRotating = true;
        for(var i=0; i<360; i++) {
            cupWithLid.transform.Rotate(Vector3.forward, 1f);
            yield return null;
        }
        cupRotating = false;
    }

    public void ShowVolume() {
        volumeCanvas.SetBool("isVisible", true);
    }

    public void HideVolume() {
        volumeCanvas.SetBool("isVisible", false);
    }

    public void ShowControls() {
        tabletCanvas.SetBool("isOn", true);
    }

    public void HideControls() {
        tabletCanvas.SetBool("isOn", false);
    }

    public void VolumeUp() {
        volup = true;
        if (volup && play) CloseInteractivityDialog();
    }

    public void PlayControl() {
        play = true;
        if (volup && play) CloseInteractivityDialog();
    }

    public void CloseInteractivityDialog() {
        interactivityCanvas.SetBool("isVisible", false);
        interactivityCanvas.gameObject.SetActive(false);
        tablet.SetActive(false);
        speaker.SetActive(false);
        closeCanvas.gameObject.SetActive(true);
        closeCanvas.SetBool("isVisible", true);
    }

    public void ShowSalirDialog() {
        hideSalirDialog.SetBool("isVisible", true);
    }

    public void HideSalirDialog() {
        hideSalirDialog.SetBool("isVisible", false);
    }

    public void LoadCinema() {
        quitting = true;
        cursorCanvas.gameObject.SetActive(false);
        StopCoroutine(LensDistanceTimeOut());
        hideSalirDialog.SetBool("isVisible", false);
        closeCanvas.gameObject.SetActive(false);
        lensAdjustCanvas.gameObject.SetActive(false);
        clickCanvas.gameObject.SetActive(false);
        interactivityCanvas.gameObject.SetActive(false);
        cupWithLid.SetActive(false);
        popcornBucket.SetActive(false);
        ticket.SetActive(false);

        loadingCanvas.SetActive(true);
        PlayerPrefs.SetInt("TutorialSuperado", 1);
        StartCoroutine(LoadAsync("Cine"));
    }

    IEnumerator LoadAsync(string scene) {
        AsyncOperation async = Application.LoadLevelAsync(scene);
        yield return async;
        Debug.Log("Loading complete");
    }

}
