﻿/*
Autor: Vicente Manuel García Huete - vicman.lobo@gmail.com
Este software está licenciado como Creative Commons Atribución-NoComercial-SinDerivar 4.0 Internacional

Quedan excluídos de la Licencia, y se prohíbe su redistribución, los siguientes recursos con Licencias Privadas:
Directorio: "Assets/EasyMovieTexture" (https://www.assetstore.unity3d.com/en/#!/content/10032)
Directorio: "Assets/Coindrop assets" (https://www.assetstore.unity3d.com/en/#!/content/28039)

Recursos con Licencias Creative Commons Utilizados:
YoutubeExtractor: https://github.com/flagbug/YoutubeExtractor
Youtube Data API v3: https://developers.google.com/youtube/v3/
Wallpaper Tablet: https://www.flickr.com/photos/101039078@N06/9759631834
Sprite de Carga: https://pixabay.com/es/proyector-proyector-cinematogr%C3%A1fico-361784/
Icono Youtube: https://pixabay.com/es/video-jugar-multimedia-youtube-481821/
Icono Imagen: https://commons.wikimedia.org/wiki/File:Image_manquante_2.svg
Iconos Zoom: https://commons.wikimedia.org/wiki/File:Zoom_out_font_awesome.svg
https://commons.wikimedia.org/wiki/File:Zoom_in_font_awesome.svg
Icono Salir: https://pixabay.com/es/registro-fuera-cerrar-sesi%C3%B3n-salida-35496/
Icono Mute: https://commons.wikimedia.org/wiki/File:Speaker_Icon.svg#/media/File:Mute_Icon.svg
Icono Volume Up: https://commons.wikimedia.org/wiki/File:Speaker_Icon.svg
Iconos Reproducción: https://pixabay.com/es/multimedia-botones-jugador-jugar-153230/
Icono Sprite Gaze: https://commons.wikimedia.org/wiki/File:Map_pointer.svg
Modelo Tablet: http://www.modelplusmodel.com/tech/apple-ipad-2.html
Icono Video: https://pixabay.com/es/cuenta-regresiva-cine-pel%C3%ADcula-155439/
Icono Preferencias: https://pixabay.com/es/engranajes-opciones-configuraci%C3%B3n-467261/
Icono Galería: https://pixabay.com/es/portapapeles-pel%C3%ADcula-clap-305778/
Fuente Cinerama: http://www.dafont.com/es/cinerama.font
*/

using UnityEngine; 
using System.Collections;
using System.Collections.Generic;
using UnityEngine.EventSystems;
using UnityEngine.UI;
using System.IO;
using System.Linq;
using Newtonsoft.Json;
using YoutubeExtractor;
using System;
using System.ComponentModel;
using System.Net;

public class UIManagerScript : MonoBehaviour {
    private const string YOUTUBE_URL = "";
    private const int SEEK_SECONDS = 15000;                         // Tiempo en milisegundos*10 que se avanza o retrocede el video
    private const int GALLERY_ITEMS = 8;                            // Numero maximo de imagenes o videos en una pagina de la galeria

    private bool isEnvironmentVisible = true;                       // Bandera que indica si el escenario 3D es visible
    private bool isEnvironmentInteractive = true;                   // Bandera que indica si el entorno 3D es interactivo
    private float videoProgress = 0f;                               // Progreso actual del video
    private FileInfo[] userVideos = { };                            // Videos locales grabados encontrados en el dispositivo
    private FileInfo[] userMovies = { };                            // Videos locales encontrados en el dispositivo
    private FileInfo[] userPhotos = { };                            // Imagenes locales encontradas en el dispositivo
    private FileInfo[] debugVideos = { };                           // Videos encontrados en el directorio de DEBUG del Editor Unity
    private FileInfo[] debugImages = { };                           // Imagenes encontradas en el directorio de DEBUG del Editor Unity
    public FileInfo[] images = { };                                // Imagenes encontradas en el dispositivo filtradas excluyendo Thumbnails
    public FileInfo[] videos = { };                                // Videos encontrados en el dispositivo, tanto si son grabados como si no
    private AndroidJavaObject unityAndroidJavaObject;               // Referencia al Contexto de la aplicacion
    private AndroidJavaClass galleryJavaClass;                      // Plugin de Android para mediar con la Galeria Multimedia del dispositivo
    private Material photoScreenMaterial;                           // Referencial al Material de la pantalla de imagenes
    private Material photo360Material;                              // Referencia al Material de la esfera de 360
    private YoutubeController youtubeController;                    // Clase encargada de la gestion de la comunicacion con la API de Youtube
    private bool initializingGalleries;                             // Bandera que indica que se esta haciendo la carga inicial de galerias de video e imagenes locales
    private DirectoryInfo galleryPath;                              // Path local de la Galeria Multimedia del dispositivo (DCIM)
    private DirectoryInfo moviesGalleryPath;                        // Path local de la Galeria de Videos del dispositivo
    private int currentChair;

    public RawImage gaze2D;                                         // Referencia al grafico 2D del Gaze o puntero de usuario
    public int video_gallery_page = 0;                              // Pagina actual de la galeria de videos locales
    public int image_gallery_page = 0;                              // Pagina actual de la galeria de imagenes locales
    public Animator mainMenuAnimator;                               // Referencia al Animator del panel de Menu Principal
    public Animator settingsAnimator;                               // Referencia al Animator del panel de Preferencias
    public Animator videoGalleryAnimator;                           // Referencia al Animator del panel de Galeria de Videos locales
    public Animator imageGalleryAnimator;                           // Referencia al Animator del panel de Galeria de Imagenes locales
    public Animator youtubeGalleryAnimator;                         // Referencia al Animator del panel de Galeria de Videos 2D de Youtube
    public Animator youtube360GalleryAnimator;                      // Referencia al Animator del panel de Galeria de Videos 360 de Youtube
    public Animator chooseYoutubeTypeAnimator;                      // Referencia al Animator del panel de Seleccion de Tipo de Videos de Youtube (2D o 360)
    public Animator chooseVideoTypeAnimator;                        // Referencia al Animator del panel de Seleccion de Tipo de Video local (2D o 360)
    public Animator chooseImageTypeAnimator;                        // Referencia al Animator del panel de Seleccion de Tipo de Imagen local (2D o 360)
    public Animator tabletAnimator;                                 // Referencia al Animator de la pantalla del tablet (Controles de Reproduccion en modo Entorno Interactivo)
    public Animator controlsAnimator;                               // Referencia al Animator del panel de Controles Unificados (situados bajo la pantalla)
    public Animator mediaSelectorAnimator;                          // Referencia al Animator del panel de Seleccion de Tipo de Medio (Imagenes, Youtube o Videos)
    public Animator controls360Animator;                            // Referencia al Animator del panel de Controles de Videos 360 (situados a los pies del usuario)
    public Animator controls360PhotoAnimator;                       // Referencia al Animator del panel de Controles de Imagenes 360 (situados a los pies del usuario)
    public Animator exitDialogAnimator;                             // Referencia al Animator del panel de Dialogo de Salida de la Aplicacion

    public GameObject cinemaEnvironment;                            // Referencia al Escenario 3D
    public GameObject unifiedControls;                              // Referencia al Objeto encargado de la gestion de los Controles Unificados (situado bajo la pantalla)
    public GameObject tablet;                                       // Referencia al Objeto del Tablet encargado de la gestion de los Controles de Reproduccion del Entorno Interactivo
    public GameObject floor;                                        // Referencia al Objeto del suelo del modo sin escenario 3D

    public Slider volumeSliderL1;                                   // Referencia al Slider de Volumen del Altavoz Izquierdo 1
    public Slider volumeSliderL2;                                   // Referencia al Slider de Volumen del Altavoz Izquierdo 2
    public Slider volumeSliderR1;                                   // Referencia al Slider de Volumen del Altavoz Derecho 1
    public Slider volumeSliderR2;                                   // Referencia al Slider de Volumen del Altavoz Derecho 2
    public Slider volumeSliderUnificados;                           // Referencia al Slider de Volumen del los Controles Unificados (situados bajo la pantalla)

    public Toggle environmentToggle;                                // Referencia al checkbox de activacion/desactivacion del escenario 3D, del panel de preferencias
    public Toggle environmentInteractiveToggle;                     // Referencia al checkbox de activacion/desactivacion de los elementos interactivos del escendario 3D, del panel de preferencias
    public Toggle youtube1440;                                      // Referencia al checkbox de preferencia de Resolucion a 1440 de Youtube, del panel de preferencias
    public Toggle youtube1080;                                      // Referencia al checkbox de preferencia de Resolucion a 1080 de Youtube, del panel de preferencias
    public Toggle youtube720;                                       // Referencia al checkbox de preferencia de Resolucion a 720 de Youtube, del panel de preferencias
    public Toggle youtube480;                                       // Referencia al checkbox de preferencia de Resolucion a 480 de Youtube, del panel de preferencias
    public Toggle youtube360;                                       // Referencia al checkbox de preferencia de Resolucion a 360 de Youtube, del panel de preferencias

    public GameObject cardboardMain;                                // Referencia al Objeto de gestion de camaras Google Cardboard
    public Transform[] chairPositions;                              // Array de Referencias de Posiciones de Butacas del usuario

    public MediaPlayerCtrl MediaPlayer;                             // Referencia al Script controlador de Video 2D
    public Image progressBarUnified;                                // Referencia a la Barra de progreso de video de los Controles Unificados (situados bajo la pantalla)
    public Image progressBarIpad;                                   // Referencia a la Barra de progreso de video de los Controles de Reproduccion Interactivos (Tablet)
    public Image progressBufferUnified;                             // Referencia a la Barra de progreso del Buffer de video de Youtube de los Controles Unificados (situados bajo la pantalla)
    public Image progressBufferIpad;                                // Referencia a la Barra de progreso del Buffer de video de Youtube de los Controles de Reproduccion Interactivos (Tablet)
    public Image progressBar360;                                    // Referencia a la Barra de progreso de video de los controles del modo video 360
    public Image progressBuffer360;                                 // Referencia a la Barra de progreso del Buffer de video de los controles del modo video 360
    public Canvas ScreenCanvas;                                     // Referencia al Canvas de pantalla (Menus)

    public GridLayoutGroup videoGalleryGrid;                        // Referencia al Grid de Videos locales
    public GridLayoutGroup imageGalleryGrid;                        // Referencia al Grid de Imagens locales
    public GridLayoutGroup youtube2DGalleryGrid;                    // Referencia al Grid de Videos 2D de Youtube
    public GridLayoutGroup youtube360GalleryGrid;                   // Referencia al Grid de Videos 360 de Youtube
    public Text selectedVideoName;                                  // Referencia al Label que muestra el nombre del video al que esta mirando el usuario
    public Text selectedImageName;                                  // Referencia al Label que muestra el nombre de la imagen a la que esta mirando el usuario
    public Text selectedYoutubeName;                                // Referencia al Label que muestra el nombre del video 2D de Youtube al que esta mirando el usuario
    public Text selectedYoutube360Name;                             // Referencia al Label que muestra el nombre del video 360 al que esta mirando el usuario
    public Text selectedVideoDurationUnified;                       // Referencia al Label que muestra la Duración del video en los Controles Unificados (situados bajo la pantalla)
    public Text selectedVideoDurationIpad;                          // Referencia al Label que muestra la Duración del video en los Controles Interactivos (tablet)
    public Text selectedVideoDuration360;                           // Referencia al Label que muestra la Duración del video en los Controles del modo 360 (situados bajo la pantalla)
    public Text selectedVideoPositionUnified;                       // Referencia al Label que muestra la Posicion de reproduccion del video en los Controles Unificados (situados bajo la pantalla)
    public Text selectedVideoPositionIpad;                          // Referencia al Label que muestra la Posicion de reproduccion del video en los Controles Interactivos (tablet)
    public Text selectedVideoPosition360;                           // Referencia al Label que muestra la Posicion de reproduccion del video en los Controles del modo 360 (situados bajo la pantalla)

    public Button nextPageButton;                                   // Referencia al Boton de avance de pagina de galerias
    public Button prevPageButton;                                   // Referencia al Boton de retroceso de pagina de galerias

    public GameObject loadingSprite;                                // Referencia al objeto de sprite de carga
    public Image loadingBar;                                        // Referencia al sprite de barra de carga

    public GameObject photoScreen;                                  // Referencia al objeto de pantalla de imagenes
    public GameObject photo360Screen;                               // Referencia al objeto de pantalla de imagenes 360
    public MediaPlayerCtrl mediaPlayer360;                          // Referencia al Script controlador de la reproduccion de videos 360

    public GameObject completeCinema;                               // Referencia al objeto que contiene el escenario y todos los elementos del modo 2D (excepto el suelo)
    public GameObject screenCanvas;                                 // Referencia al objeto del canvas de pantalla (menu)

    public string selectedMediaPath;                                // Cadena que contendra el path fisico o url del video o imagen seleccionado por el usuario

    public Sprite noThumbnailSprite;                                // Sprite usado en caso de no tener previsualizacion para una imagen o video

    Action OnStop;                                                  // Funcion que se ejecuta al detener un video de forma manual o por haber llegado a su fin (usado para mostrar una galeria determinada en el canvas)
    

    public void Start() {
#if UNITY_ANDROID && !UNITY_EDITOR
        // Obtenemos el Contexto Android de la aplicacion
        AndroidJavaClass unityAndroidJavaClass = new AndroidJavaClass("com.unity3d.player.UnityPlayer"); 
        unityAndroidJavaObject = unityAndroidJavaClass.GetStatic<AndroidJavaObject>("currentActivity");
        galleryJavaClass = new AndroidJavaClass("org.vmgarcia.gallerymediator.GalleryManager");
#endif


        youtubeController = GetComponent<YoutubeController>();
        photoScreenMaterial = photoScreen.GetComponent<Renderer>().material;
        photo360Material = photo360Screen.GetComponent<Renderer>().material;

        RefreshVolumeBars();

        if (PlayerPrefs.HasKey("Environment")) {
            isEnvironmentVisible = (PlayerPrefs.GetInt("Environment") > 0);
        } else {
            isEnvironmentVisible = true;
            PlayerPrefs.SetInt("Environment", 1);
        }

        if (PlayerPrefs.HasKey("InteractiveEnvironment")) {
            isEnvironmentInteractive = (PlayerPrefs.GetInt("InteractiveEnvironment") > 0);
        } else {
            isEnvironmentInteractive = true;
            PlayerPrefs.SetInt("InteractiveEnvironment", 1);
        }

        // Recupera las preferencias de resolucion de videos de youtube
        if (PlayerPrefs.GetInt("YoutubeResolution") == 0) PlayerPrefs.GetInt("YoutubeResolution", 720);
        int youtubeRes = PlayerPrefs.GetInt("YoutubeResolution");
        environmentToggle.isOn = isEnvironmentVisible;
        environmentInteractiveToggle.isOn = isEnvironmentInteractive;
        youtube1440.isOn = (youtubeRes == 1440);
        youtube1080.isOn = (youtubeRes == 1080);
        youtube720.isOn = (youtubeRes == 720);
        youtube480.isOn = (youtubeRes == 480);
        youtube360.isOn = (youtubeRes == 360);

        // Recupera las preferencias de escenario 3D y/o elementos interactivos
        TurnOnEnvironment(isEnvironmentVisible);
        TurnOnInteractiveEnvironment(isEnvironmentInteractive);

        // Inicializa las barras de progreso de reproduccion de video
        progressBarIpad.fillAmount = 0;
        progressBarUnified.fillAmount = 0;
        progressBufferIpad.fillAmount = 0;
        progressBufferUnified.fillAmount = 0;

        // Bandera que indica que se estan cargando las galerias de videos e imagenes locales
        initializingGalleries = true;

        // Recupera la preferencia de butaca del usuario
        currentChair = PlayerPrefs.GetInt("Butaca");
        if (currentChair == 0) currentChair = 6;
        JumpToChair(currentChair);

        // Inicia el escaneo asincrono de videos e imagenes locales
        StartCoroutine(MediaScan());
    }

    // Corrutina que actualiza, cada segundo, las barras de progreso, el tiempo de reproduccion y la duracion del video 2D
    IEnumerator Update2DVideoProgress() {
        // Mientras la pantalla de reproduccion este activa
        while(MediaPlayer.gameObject.activeSelf) {
            if (MediaPlayer.GetDuration() > 0) {
                float position = (float)(MediaPlayer.GetSeekPosition() / 1000);
                float duration = (float)(MediaPlayer.GetDuration() / 1000);
                videoProgress = position / duration;
                progressBarUnified.fillAmount = videoProgress;
                progressBarIpad.fillAmount = videoProgress;

                TimeSpan t = TimeSpan.FromSeconds(duration);
                selectedVideoDurationIpad.text = string.Format("{0:D2}:{1:D2}:{2:D2}", t.Hours, t.Minutes, t.Seconds);
                selectedVideoDurationUnified.text = selectedVideoDurationIpad.text;
                t = TimeSpan.FromSeconds(position);
                selectedVideoPositionIpad.text = string.Format("{0:D2}:{1:D2}:{2:D2}", t.Hours, t.Minutes, t.Seconds);
                selectedVideoPositionUnified.text = selectedVideoPositionIpad.text;
            }

            progressBufferUnified.fillAmount = MediaPlayer.GetCurrentSeekPercent();
            progressBufferIpad.fillAmount = MediaPlayer.GetCurrentSeekPercent();

            // Espera un segundo hasta la proxima ejecucion
            yield return new WaitForSeconds(1);
        }
    }

    // Corrutina que actualiza, cada segundo, la barra de progreso, el tiempo de reproduccion y la duracion del video 360
    IEnumerator Update360VideoProgress() {
        // Mientras la esfera 360 este activa
        while (mediaPlayer360.gameObject.activeSelf) {
            if (mediaPlayer360.GetDuration() > 0) {
                float position = (float)(mediaPlayer360.GetSeekPosition() / 1000);
                float duration = (float)(mediaPlayer360.GetDuration() / 1000);
                videoProgress = position / duration;
                progressBar360.fillAmount = videoProgress;

                TimeSpan t = TimeSpan.FromSeconds(duration);
                selectedVideoDuration360.text = string.Format("{0:D2}:{1:D2}:{2:D2}", t.Hours, t.Minutes, t.Seconds);
                t = TimeSpan.FromSeconds(position);
                selectedVideoPosition360.text = string.Format("{0:D2}:{1:D2}:{2:D2}", t.Hours, t.Minutes, t.Seconds);
            }

            progressBuffer360.fillAmount = mediaPlayer360.GetCurrentSeekPercent();

            // Espera un segundo hasta la proxima ejecucion
            yield return new WaitForSeconds(1);
        }
    }


    // Muestra el panel de configuracion y oculta menu principal
    public void ShowSettings () {
        mainMenuAnimator.SetBool("isVisible", false);
        settingsAnimator.SetBool("isVisible", true);
    }

    // Oculta el panel de configuracion y muestra menu principal
    public void HideSettings () {
        mainMenuAnimator.SetBool("isVisible", true);
        settingsAnimator.SetBool("isVisible", false);
    }

    // Muestra la galeria de videos 2D y oculta el selector de medios
    public void ShowVideoGallery () {
        mediaSelectorAnimator.SetBool("isVisible", false);
        videoGalleryAnimator.SetBool("isVisible", true);
        if (video_gallery_page < (videos.Length/GALLERY_ITEMS)) nextPageButton.gameObject.SetActive(true);
        if (video_gallery_page > 0) prevPageButton.gameObject.SetActive(true);
    }

    // Oculta la galeria de videos 2D y muestra el selector de medios
    public void HideVideoGallery () {
        mediaSelectorAnimator.SetBool("isVisible", true);
        videoGalleryAnimator.SetBool("isVisible", false);
        nextPageButton.gameObject.SetActive(false);
        prevPageButton.gameObject.SetActive(false);
    }

    // Muestra la galeria de imagenes y oculta el selector de medios
    public void ShowImageGallery() {
        mediaSelectorAnimator.SetBool("isVisible", false);
        imageGalleryAnimator.SetBool("isVisible", true);
        if (image_gallery_page < (images.Length / GALLERY_ITEMS)) nextPageButton.gameObject.SetActive(true);
        if (image_gallery_page > 0) prevPageButton.gameObject.SetActive(true);
    }

    // Oculta la galeria de imagenes y muestra el selector de medios
    public void HideImageGallery() {
        mediaSelectorAnimator.SetBool("isVisible", true);
        imageGalleryAnimator.SetBool("isVisible", false);
        nextPageButton.gameObject.SetActive(false);
        prevPageButton.gameObject.SetActive(false);
    }

    // Muestra la galeria de videos 2D de Youtube y oculta el selector de tipo de videos de Youtube (2D o 360)
    // En caso de no estar cargada ninguna pagina de videos de Youtube, lanza la corrutina para consultar la lista de videos de youtube
    public void ShowYoutube2DGallery() {
        Debug.Log(youtube2DGalleryGrid.transform.GetChild(0).gameObject.activeSelf);
        Debug.Log(youtubeController.next2DPage == "");
        Debug.Log(youtubeController.prev2DPage == "");
        if (!youtube2DGalleryGrid.transform.GetChild(0).gameObject.activeSelf && (youtubeController.next2DPage == null || youtubeController.next2DPage == "") && (youtubeController.prev2DPage == null || youtubeController.prev2DPage == "")) {
            chooseYoutubeTypeAnimator.SetBool("isVisible", false);
            StartCoroutine(youtubeController.MediaScan("", false, (value) => StartCoroutine(LoadYoutube2DGalleryPage(value, true))));
        } else {
            chooseYoutubeTypeAnimator.SetBool("isVisible", false);
            youtubeGalleryAnimator.SetBool("isVisible", true);
            nextPageButton.gameObject.SetActive(youtubeController.next2DPage != null);
            prevPageButton.gameObject.SetActive(youtubeController.prev2DPage != null);
        }
    }

    // Oculta la galeria de videos 2D de Youtube y muestra el selector de tipo de videos de Youtube (2D o 360)
    public void HideYoutube2DGallery() {
        chooseYoutubeTypeAnimator.SetBool("isVisible", true);
        youtubeGalleryAnimator.SetBool("isVisible", false);
        nextPageButton.gameObject.SetActive(false);
        prevPageButton.gameObject.SetActive(false);
    }

    // Muestra la galeria de videos 360 de Youtube y oculta el selector de tipo de videos de Youtube (2D o 360)
    public void ShowYoutube360Gallery() {
        if (!youtube360GalleryGrid.transform.GetChild(0).gameObject.activeSelf && (youtubeController.next360Page == null || youtubeController.next360Page == "") && (youtubeController.prev360Page == null || youtubeController.prev360Page == "")) {
            chooseYoutubeTypeAnimator.SetBool("isVisible", false);
            StartCoroutine(youtubeController.MediaScan("", true, (value) => StartCoroutine(LoadYoutube360GalleryPage(value, true))));
        } else {
            chooseYoutubeTypeAnimator.SetBool("isVisible", false);
            youtube360GalleryAnimator.SetBool("isVisible", true);
            nextPageButton.gameObject.SetActive(youtubeController.next360Page != null);
            prevPageButton.gameObject.SetActive(youtubeController.prev360Page != null);
        }
    }

    // Oculta la galeria de videos 360 de Youtube y muestra el selector de tipo de videos de Youtube (2D o 360)
    public void HideYoutube360Gallery() {
        chooseYoutubeTypeAnimator.SetBool("isVisible", true);
        youtube360GalleryAnimator.SetBool("isVisible", false);
        nextPageButton.gameObject.SetActive(false);
        prevPageButton.gameObject.SetActive(false);
    }

    // Muestra el selector de medios y oculta el menu principal
    public void ShowMediaSelector() {
        mediaSelectorAnimator.SetBool("isVisible", true);
        mainMenuAnimator.SetBool("isVisible", false);
    }

    // Oculta el selector de medios y muestra el menu principal
    public void HideMediaSelector() {
        mediaSelectorAnimator.SetBool("isVisible", false);
        mainMenuAnimator.SetBool("isVisible", true);
    }

    // Muestra el selector de tipo de video de Youtube (2D o 360) y oculta el selector de medios
    public void ShowYoutubeTypeSelector() {
        chooseYoutubeTypeAnimator.SetBool("isVisible", true);
        mediaSelectorAnimator.SetBool("isVisible", false);
    }

    // Oculta el selector de tipo de video de Youtube (2D o 360) y muestra el selector de medios
    public void HideYoutubeTypeSelector() {
        chooseYoutubeTypeAnimator.SetBool("isVisible", false);
        mediaSelectorAnimator.SetBool("isVisible", true);
    }

    // Muestra el selector de tipo de video local (2D o 360) y oculta la galeria de videos locales
    public void ShowVideoTypeSelector(Text path) {
        selectedMediaPath = path.text;
        chooseVideoTypeAnimator.SetBool("isVisible", true);
        videoGalleryAnimator.SetBool("isVisible", false);
    }

    // Oculta el selector de tipo de video local (2D o 360) y muestra la galeria de videos locales
    public void HideVideoTypeSelector() {
        selectedMediaPath = "";
        chooseVideoTypeAnimator.SetBool("isVisible", false);
        videoGalleryAnimator.SetBool("isVisible", true);
    }

    // Muestra el selector de tipo de imagen local (2D o 360) y oculta la galeria de imagenes locales
    public void ShowImageTypeSelector(Text path) {
        selectedMediaPath = path.text;
        chooseImageTypeAnimator.SetBool("isVisible", true);
        imageGalleryAnimator.SetBool("isVisible", false);
    }

    // Oculta el selector de tipo de imagen local (2D o 360) y muestra la galeria de imagenes locales
    public void HideImageTypeSelector() {
        selectedMediaPath = "";
        chooseImageTypeAnimator.SetBool("isVisible", false);
        imageGalleryAnimator.SetBool("isVisible", true);
    }

    // Muestra los controles de reproduccion de video del modo 360, situados a los pies del usuario
    public void TurnOn360Controls() {
        controls360Animator.SetBool("isOn", true);
    }

    // Oculta los controles de reproduccion de video del modo 360, situados a los pies del usuario
    public void TurnOff360Controls() {
        controls360Animator.SetBool("isOn", false);
    }

    // Muestra los controles de reproduccion de imagen del modo 360, situados a los pies del usuario
    public void TurnOn360PhotoControls() {
        controls360PhotoAnimator.SetBool("isOn", true);
    }

    // Oculta los controles de reproduccion de imagen del modo 360, situados a los pies del usuario
    public void TurnOff360PhotoControls() {
        controls360PhotoAnimator.SetBool("isOn", false);
    }

    // Muestra la pantalla del tablet, con los controles de reproduccion de video e imagen del modo de entorno interactivo
    public void TurnOnTablet() {
        tabletAnimator.SetBool("isOn", true);
    }

    // Oculta la pantalla del tablet, con los controles de reproduccion de video e imagen del modo de entorno interactivo
    public void TurnOffTablet() {
        tabletAnimator.SetBool("isOn", false);
    }

    // Muestra los controles de Volumen del sonido en el mode de entorno interactivo
    public void ShowVolumeControls(GameObject speaker) {
        if (isEnvironmentInteractive) speaker.GetComponent<Animator>().SetBool("isVisible", true);
    }

    // Oculta los controles de Volumen del sonido en el modo de entorno interactivo
    public void HideVolumeControls(GameObject speaker) {
        if (isEnvironmentInteractive) speaker.GetComponent<Animator>().SetBool("isVisible", false);
    }

    // Muestra el dialogo de salida de la aplicacion en el modo de entorno interactivo
    public void ShowExit(GameObject door) {
        if (isEnvironmentInteractive) door.GetComponent<Animator>().SetBool("isVisible", true);
    }

    // Oculta el dialogo de salida de la aplicacion en el modo de entorno interactivo
    public void HideExit(GameObject door) {
        if (isEnvironmentInteractive) {
            door.GetComponent<Animator>().SetBool("isVisible", false);
        }
    }

    // Muestra el dialogo de confirmacion de salida de la aplicacion tanto para entorno interactivo como unificado
    public void ShowExitDialog() {
        screenCanvas.GetComponent<Canvas>().enabled = false;
        exitDialogAnimator.SetBool("isVisible", true);
    }

    // Oculta el dialogo de confirmacion de salida de la aplicacion tanto para entorno interactivo como unificado
    public void HideExitDialog() {
        screenCanvas.GetComponent<Canvas>().enabled = true;
        exitDialogAnimator.SetBool("isVisible", false);
    }

    // Muestra el tutorial
    public void ShowTutorial() {
        PlayerPrefs.SetInt("TutorialSuperado", 0);
        loadingSprite.gameObject.SetActive(true);
        mainMenuAnimator.SetBool("isVisible", false);
        StartCoroutine(LoadAsync("Tutorial"));
    }

    // Carga asincrona de escena
    IEnumerator LoadAsync(string scene) {
        UnityEngine.AsyncOperation async = Application.LoadLevelAsync(scene);
        yield return async;
        Debug.Log("Loading complete");
    }

    // Sube el volumen de audio 
    public void VolumeUp() {
        AudioListener.volume += 0.1f;
        RefreshVolumeBars();
        #if UNITY_ANDROID && !UNITY_EDITOR
        galleryJavaClass.CallStatic("VolumeUp", unityAndroidJavaObject);
        #endif
    }

    // Baja el volumen de audio 
    public void VolumeDown() {
        AudioListener.volume -= 0.1f;
        RefreshVolumeBars();
        #if UNITY_ANDROID && !UNITY_EDITOR
        galleryJavaClass.CallStatic("VolumeDown", unityAndroidJavaObject);
        #endif
    }

    // Silencia o Activa el audio
    public void VolumeMute() {
        AudioListener.pause = !AudioListener.pause;
        volumeSliderL1.enabled = AudioListener.pause;
        volumeSliderL2.enabled = AudioListener.pause;
        volumeSliderR1.enabled = AudioListener.pause;
        volumeSliderR2.enabled = AudioListener.pause;
        volumeSliderUnificados.enabled = AudioListener.pause;
        #if UNITY_ANDROID && !UNITY_EDITOR
        galleryJavaClass.CallStatic("VolumeMute", unityAndroidJavaObject, AudioListener.pause);
        #endif
    }

    // Establece el volumen del audio
    public void VolumeSet(float volume) {
        AudioListener.volume = volume;
        RefreshVolumeBars();
        #if UNITY_ANDROID && !UNITY_EDITOR
        galleryJavaClass.CallStatic("VolumeTo", unityAndroidJavaObject, volume);
        #endif
    }

    // Actualiza el valor de volumen del audio en todos los controles 'Slider' tanto de entorno interactivo como unificado
    private void RefreshVolumeBars() {
        volumeSliderL1.value = AudioListener.volume;
        volumeSliderL2.value = AudioListener.volume;
        volumeSliderR1.value = AudioListener.volume;
        volumeSliderR2.value = AudioListener.volume;
        volumeSliderUnificados.value = AudioListener.volume;
    }

    // Cierra la aplicacion
    public void Quit() {
        Application.Quit();
    }

    // Muestra los controles unificados situados bajo la pantalla
    public void ShowControls() {
        controlsAnimator.SetBool("isVisible", true);
    }

    // Oculta los controles unificados situados bajo la pantalla
    public void HideControls() {
        controlsAnimator.SetBool("isVisible", false);
    }

    // Activa o desactiva el entorno de 360 grados, el cine, el suelo, la esfera 360 y los elementos interactivos si estan configurados
    public void TurnOn360Environment(bool isActive) {
        if (isActive) {
            completeCinema.SetActive(false);
            floor.SetActive(false);
            tablet.SetActive(false);
            screenCanvas.SetActive(false);
        } else {
            completeCinema.SetActive(true);
            floor.SetActive(true);
            screenCanvas.SetActive(true);
            if (PlayerPrefs.GetInt("InteractiveEnvironment") > 0) tablet.SetActive(true);
        }
    }


    // Activa o desactiva el escenario 3D (cine) y por consiguiente los elementos interactivos
    public void TurnOnEnvironment(bool isActive) {
        PlayerPrefs.SetInt("Environment", (isActive ? 1 : 0));
        isEnvironmentVisible = isActive;
        cinemaEnvironment.active = isActive;
        if (!isActive) {
            environmentInteractiveToggle.isOn = false;
        }
    }

    // Activa o desactivo los elementos interactivos del escenario 3D (cine)
    public void TurnOnInteractiveEnvironment(bool isInteractive) {
        PlayerPrefs.SetInt("InteractiveEnvironment", (isInteractive ? 1 : 0));
        isEnvironmentInteractive = isInteractive;
        unifiedControls.active = !isInteractive;
        tablet.active = isInteractive;
    }

    // Establece la preferencia de resolucion de videos de youtube seleccionada en el panel de configuracion
    public void SetYoutubeResolution(Text resolution) {
        PlayerPrefs.SetInt("YoutubeResolution", Int32.Parse(resolution.text));
    }

    // Mueve al espectador a la butaca seleccionada y guarda la seleccion en preferencias del usuario
    public void JumpToChair(int chairNumber) {
        currentChair = chairNumber;
        cardboardMain.transform.position = chairPositions[chairNumber].position;
        PlayerPrefs.SetInt("Butaca", currentChair);
    }

    // Aleja al espectador una butaca mas lejos de la pantalla
    public void NextChair() {
        if (currentChair < 9) currentChair++;
        JumpToChair(currentChair);
    }

    // Acerca al espectador una butaca mas cerca de la pantalla
    public void PrevChair() {
        if (currentChair > 0) currentChair--;
        JumpToChair(currentChair);
    }
    
    // Inicia la reproduccion de un video 2D o 360, ocultando el canvas previamente
    public void PlayVideo() {
        if (MediaPlayer.gameObject.activeSelf) {
            ScreenCanvas.gameObject.SetActive(false);
            MediaPlayer.Play();
            StartCoroutine(Update2DVideoProgress());
        }
        if (mediaPlayer360.gameObject.activeSelf) {
            ScreenCanvas.gameObject.SetActive(false);
            mediaPlayer360.Play();
            StartCoroutine(Update360VideoProgress());
        }
    }

    // Muestra una foto previamente cargada y oculta el canvas
    public void ShowPhoto() {
        photoScreen.SetActive(true);
        ScreenCanvas.gameObject.SetActive(false);
    }

    // Detiene definitivamente la reproduccion de un video 2D o 360, deja a cero las barras de progreso, duracion y posicion, y muestra el canvas del menu
    public void StopVideo() {
        if (MediaPlayer.gameObject.activeSelf) {
            MediaPlayer.Stop();
            MediaPlayer.UnLoad();
            MediaPlayer.gameObject.SetActive(false); // Detiene la corrutina Update2dVideoProgress
            ScreenCanvas.gameObject.SetActive(true);
            progressBarIpad.fillAmount = 0;
            progressBarUnified.fillAmount = 0;
            progressBufferIpad.fillAmount = 0;
            progressBufferUnified.fillAmount = 0;
            selectedVideoDurationIpad.text = "0:00:00";
            selectedVideoDurationUnified.text = "0:00:00";
            selectedVideoPositionIpad.text = "0:00:00";
            selectedVideoPositionUnified.text = "0:00:00";
        }

        if (photoScreen.activeSelf) {
            photoScreen.SetActive(false);
            ScreenCanvas.gameObject.SetActive(true);
        }

        if (photo360Screen.activeSelf) {
            ScreenCanvas.gameObject.SetActive(true);
            photo360Screen.SetActive(false);
            TurnOn360Environment(false);
        }

        if (mediaPlayer360.gameObject.activeSelf) {
            ScreenCanvas.gameObject.SetActive(true);
            mediaPlayer360.Stop();
            mediaPlayer360.UnLoad();
            mediaPlayer360.gameObject.SetActive(false); // Detiene la corrutina Update360VideoProgress
            selectedVideoDuration360.text = "0:00:00";
            selectedVideoPosition360.text = "0:00:00";
            TurnOn360Environment(false);

        }
        if (!gaze2D.enabled) gaze2D.enabled = true; // Muestra el Gaze en caso de que este estuviera oculto por mirar a pantalla
        if (OnStop != null) OnStop();   // Vuelve al panel del canvas previamente seleccionado (su funcion Show...)
    }
   

    // Detiene momentaneamente el video 2D o 360, parando la corrutina de refresco de barras de progreso, duracion y posicion
    public void PauseVideo() {
        if (MediaPlayer.gameObject.activeSelf) {
            MediaPlayer.Pause();
            StopCoroutine(Update2DVideoProgress());
        }
        if (mediaPlayer360.gameObject.activeSelf) {
            mediaPlayer360.Pause();
            StopCoroutine(Update360VideoProgress());
        }
    }

    // Avanza el video 2D o 360, la cantidad de segundos configurada en la constante SEEK_SECONDS
    public void ForwardVideo() {
        if (MediaPlayer.gameObject.activeSelf) MediaPlayer.SeekTo(MediaPlayer.GetSeekPosition() + SEEK_SECONDS);
        if (mediaPlayer360.gameObject.activeSelf) mediaPlayer360.SeekTo(MediaPlayer.GetSeekPosition() + SEEK_SECONDS);
    }

    // Retrocede el video 2D o 360, la cantidad de segundos configurada en la constante SEEK_SECONDS
    public void RewindVideo() {
        if (MediaPlayer.gameObject.activeSelf) MediaPlayer.SeekTo(MediaPlayer.GetSeekPosition() - SEEK_SECONDS);
        if (mediaPlayer360.gameObject.activeSelf) mediaPlayer360.SeekTo(MediaPlayer.GetSeekPosition() - SEEK_SECONDS);
    }

    // Inicia la carga de un video 2D local, establece el panel de galeria de videos locales como panel de vuelta y reproduce el video
    public void Load2DVideo() {
        OnStop = ShowVideoGallery;
        nextPageButton.gameObject.SetActive(false);
        prevPageButton.gameObject.SetActive(false);
        MediaPlayer.Load("file://" + selectedMediaPath);
        MediaPlayer.gameObject.SetActive(true);
        MediaPlayer.OnEnd = StopVideo;
        PlayVideo();
    }

    // Inicia la carga de un video 360 local, establece el panel de galeria de videos locales como panel de vuelta, activa el entorno 360 y reproduce el video
    public void Load360Video() {
        OnStop = ShowVideoGallery;
        nextPageButton.gameObject.SetActive(false);
        prevPageButton.gameObject.SetActive(false);
        mediaPlayer360.Load("file://" + selectedMediaPath);
        mediaPlayer360.gameObject.SetActive(true);
        mediaPlayer360.OnEnd = StopVideo;
        TurnOn360Environment(true);
        loadingSprite.SetActive(false);
        PlayVideo();
    }


    // Inica la carga de una imagen 2D local, establece el panel de galeria de imagenes locales como panel de vuelta y muestra la imagen
    public void Load2DPhoto() {
        OnStop = ShowImageGallery;
        loadingBar.fillAmount = 0;
        loadingSprite.gameObject.SetActive(true);
        nextPageButton.gameObject.SetActive(false);
        prevPageButton.gameObject.SetActive(false);

        byte[] data = File.ReadAllBytes(selectedMediaPath);
        loadingBar.fillAmount = 0.3f;
        Texture2D texture = new Texture2D(2, 2);
        texture.LoadImage(data);
        loadingBar.fillAmount = 0.7f;
        texture.name = Path.GetFileNameWithoutExtension(selectedMediaPath);
        photoScreenMaterial.mainTexture = texture;
        loadingBar.fillAmount = 1f;
        ShowPhoto();
        loadingSprite.gameObject.SetActive(false);
    }

    // Inica la carga de una imagen 360 local, establece el panel de galeria de imagenes locales como panel de vuelta, activa el entorno 360 y muestra la imagen
    public void Load360Photo() {
        OnStop = ShowImageGallery;
        loadingBar.fillAmount = 0;
        loadingSprite.gameObject.SetActive(true);
        nextPageButton.gameObject.SetActive(false);
        prevPageButton.gameObject.SetActive(false);

        byte[] data = File.ReadAllBytes(selectedMediaPath);
        loadingBar.fillAmount = 0.3f;
        Texture2D texture = new Texture2D(2, 2);
        texture.LoadImage(data);
        loadingBar.fillAmount = 0.7f;
        texture.name = Path.GetFileNameWithoutExtension(selectedMediaPath);
        photo360Material.mainTexture = texture;
        loadingBar.fillAmount = 1f;
        loadingSprite.gameObject.SetActive(false);
        photo360Screen.SetActive(true);
        TurnOn360Environment(true);
    }



    // Inicia la carga asincrona de un video 2D de youtube, oculta la galeria de videos 2D de youtube y reproduce el video
    public void LoadYoutube2DVideo(Text path) {
        youtubeGalleryAnimator.SetBool("isVisible", false);
        loadingBar.fillAmount = 0;
        loadingSprite.gameObject.SetActive(true);
        nextPageButton.gameObject.SetActive(false);
        prevPageButton.gameObject.SetActive(false);
        string videoId = path.text;

        IEnumerable<VideoInfo> videoInfos = DownloadUrlResolver.GetDownloadUrls(YoutubeController.YOUTUBE_VIDEO_URL + videoId, false);
        int preferedResolution = PlayerPrefs.GetInt("YoutubeResolution");
        IEnumerable<VideoInfo> videosMp4 = videoInfos.Where(vi => vi.VideoExtension == ".mp4" && vi.Resolution <= preferedResolution).OrderByDescending(vi => vi.Resolution);
        
        OnStop = ShowYoutube2DGallery;
        loadingSprite.SetActive(false);
        MediaPlayer.Load(videosMp4.First().DownloadUrl);
        MediaPlayer.gameObject.SetActive(true);
        MediaPlayer.OnEnd = StopVideo;

        PlayVideo();
    }

    // Inicia la carga asincrona de un video 360 de youtube, oculta la galeria de videos 360 de youtube y reproduce el video
    public void LoadYoutube360Video(Text path) {
        youtube360GalleryAnimator.SetBool("isVisible",false);
        loadingBar.fillAmount = 0;
        loadingSprite.gameObject.SetActive(true);
        nextPageButton.gameObject.SetActive(false);
        prevPageButton.gameObject.SetActive(false);
        string videoId = path.text;

        IEnumerable<VideoInfo> videoInfos = DownloadUrlResolver.GetDownloadUrls(YoutubeController.YOUTUBE_VIDEO_URL + videoId, false);
        int preferedResolution = PlayerPrefs.GetInt("YoutubeResolution");
        IEnumerable<VideoInfo> videosMp4 = videoInfos.Where(vi => vi.VideoExtension == ".mp4" && vi.Resolution <= preferedResolution).OrderByDescending(vi => vi.Resolution);

        OnStop = ShowYoutube360Gallery;
        mediaPlayer360.Load(videosMp4.First().DownloadUrl);
        TurnOn360Environment(true);
        loadingSprite.SetActive(false);
        mediaPlayer360.gameObject.SetActive(true);
        mediaPlayer360.OnEnd = StopVideo;

        PlayVideo();
    }


    // Asigna la textura de una imagen a la pantalla de imagenes y muestra la imagen
    public void AssignTextureToScreen(Texture2D texture) {
        photoScreenMaterial.mainTexture = texture;
        ShowPhoto();
        loadingSprite.gameObject.SetActive(false);
    }

    // Muestra el nombre de un video al que el usuario esta mirando
    public void ShowFileName(Text filename) {
        if (videoGalleryAnimator.GetBool("isVisible")) selectedVideoName.text = filename.text;
        if (imageGalleryAnimator.GetBool("isVisible")) selectedImageName.text = filename.text;
        if (youtubeGalleryAnimator.GetBool("isVisible")) selectedYoutubeName.text = filename.text;
        if (youtube360GalleryAnimator.GetBool("isVisible")) selectedYoutube360Name.text = filename.text;
    }

    // Oculta el nombre del ultimo video al que el usuario estaba mirando
    public void ClearFileName() {
        selectedVideoName.text = "";
        selectedImageName.text = "";
        selectedYoutubeName.text = "";
    }

    // Escanea, de forma asincrona, los videos e imagenes locales de las galerias del movil, utilizando los paths de galerias devueltas por el plugin Android 'GalleryMediator' 
    // y crea, de forma asincrona, la primera pagina de las galerias de videos e imagenes locales
    IEnumerator MediaScan() {
#if UNITY_ANDROID && !UNITY_EDITOR
        // Obtenemos el path y los videos grabados en el movil por el usuario
        galleryPath = new DirectoryInfo(galleryJavaClass.CallStatic<string>("getGalleryPath"));
        userVideos = galleryPath.GetFiles("*.mp4", SearchOption.AllDirectories);

        yield return null;

        // Obtenemos el path y las fotos tomadas en el movil por el usuario
        userPhotos = galleryPath.GetFiles("*.jpg", SearchOption.AllDirectories);

        yield return null;

        // Obtenemos el path y los vídeos añadidos al movil por el usuario
        moviesGalleryPath = new DirectoryInfo(galleryJavaClass.CallStatic<string>("getMoviesPath"));
        userMovies = moviesGalleryPath.GetFiles("*.mp4", SearchOption.AllDirectories);

        yield return null;

        videos = new FileInfo[userVideos.Length + userMovies.Length];
        userVideos.CopyTo(videos, 0);
        userMovies.CopyTo(videos, userVideos.Length);

        yield return null;

        // Excluimos las imágenes del directorio ".thumbnails"
        images = (from fi in userPhotos where !Path.GetDirectoryName(fi.FullName).Contains(".thumbnails") select fi).Cast<FileInfo>().ToArray();
        
#elif UNITY_EDITOR
        // Obtenemos los videos locales de DEBUG
        DirectoryInfo directoryInfo = new DirectoryInfo("c:/vv360");
        debugVideos = directoryInfo.GetFiles("*.mp4", SearchOption.AllDirectories);

        yield return null;

        // Obtenemos las imagenes locales de DEBUG
        directoryInfo = new DirectoryInfo("c:/vv360");
        debugImages = directoryInfo.GetFiles("*.jpg", SearchOption.AllDirectories);
        //Debug.Log("Número de Imágenes: "+debugImages.Length);

        yield return null;

        videos = debugVideos;
        images = debugImages;

        // Excluimos las imágenes del directorio ".thumbnails"
        images = (from fi in debugImages where !Path.GetDirectoryName(fi.FullName).Contains(".thumbnails") select fi).Cast<FileInfo>().ToArray();
#endif

        // Añadimos los GALLERY_ITEMS primeros botones a la galeria
        yield return StartCoroutine(LoadVideoGalleryPage());
        yield return StartCoroutine(LoadImageGalleryPage());
    }

    // Avanza una pagina de las galerias de imagenes y videos locales y de youtube, de forma asincrona
    public void NextPage() {
        if (!prevPageButton.gameObject.activeSelf) prevPageButton.gameObject.SetActive(true);

        if (videoGalleryAnimator.GetBool("isVisible")) {
            video_gallery_page += 1;
            StartCoroutine("LoadVideoGalleryPage");
            if (video_gallery_page >= (videos.Length / GALLERY_ITEMS)) nextPageButton.gameObject.SetActive(false);
        }

        if (imageGalleryAnimator.GetBool("isVisible")) {
            image_gallery_page += 1;
            StartCoroutine("LoadImageGalleryPage");
            if (image_gallery_page >= (images.Length / GALLERY_ITEMS)) nextPageButton.gameObject.SetActive(false);
        }

        if (youtubeGalleryAnimator.GetBool("isVisible")) {
            StartCoroutine(youtubeController.MediaScan(youtubeController.next2DPage, false, (value) => StartCoroutine(LoadYoutube2DGalleryPage(value, false))));
        }

        if (youtube360GalleryAnimator.GetBool("isVisible")) {
            StartCoroutine(youtubeController.MediaScan(youtubeController.next360Page, true, (value) => StartCoroutine(LoadYoutube360GalleryPage(value, false))));
        }


    }

    // Retrocede una pagina de las galerias de imagenes y videos locales y de youtube, de forma asincrona
    public void PrevPage() {
        if (!nextPageButton.gameObject.activeSelf) nextPageButton.gameObject.SetActive(true);

        if (videoGalleryAnimator.GetBool("isVisible")) {
            video_gallery_page -= 1;
            StartCoroutine("LoadVideoGalleryPage");
        }

        if (imageGalleryAnimator.GetBool("isVisible")) {
            image_gallery_page -= 1;
            StartCoroutine("LoadImageGalleryPage");
        }

        if (youtubeGalleryAnimator.GetBool("isVisible")) {
            StartCoroutine(youtubeController.MediaScan(youtubeController.prev2DPage, false, (value) => StartCoroutine(LoadYoutube2DGalleryPage(value, false))));
        }

        if (youtube360GalleryAnimator.GetBool("isVisible")) {
            StartCoroutine(youtubeController.MediaScan(youtubeController.prev360Page, true, (value) => StartCoroutine(LoadYoutube360GalleryPage(value, false))));
        }

    }

    // Carga de forma asincrona una pagina de la galeria de videos locales
    IEnumerator LoadVideoGalleryPage() {
        videoGalleryGrid.GetComponent<CanvasGroup>().alpha = 0;
        loadingBar.fillAmount = 0;
        loadingSprite.SetActive(true);

        foreach (Transform child in videoGalleryGrid.transform) {
            child.gameObject.SetActive(false);
            yield return null;
        }

        int i;
        int x;
        for (i = video_gallery_page * GALLERY_ITEMS, x = 0; x < GALLERY_ITEMS && i < videos.Length; i++, x++) {
            if (videoGalleryGrid.transform.childCount > x) {
                var button = videoGalleryGrid.transform.GetChild(x).GetComponent<Button>();
                button.transform.FindChild("Path").GetComponent<Text>().text = videos[i].FullName;
                button.transform.FindChild("Filename").GetComponent<Text>().text = videos[i].Name;
                button.transform.SetParent(videoGalleryGrid.transform, false);

#if UNITY_ANDROID && !UNITY_EDITOR
                // Obtenemos el Thumbnail de Android
                string thumbPath = galleryJavaClass.CallStatic<string>("getVideoThumbnailPath", unityAndroidJavaObject, videos[i].FullName);
                if (Debug.isDebugBuild) 
                Debug.Log("[THUMB] IMAGE: "+videos[i].FullName+" THUMBNAIL: "+thumbPath);
                yield return null;

                // Pintamos Thumbnail en el Boton de la Galeria/grid    
                if (thumbPath != "") {
                    Texture2D texture = new Texture2D(2, 2);
                    yield return StartCoroutine(LoadThumbnailAsset("file://"+thumbPath, value => button.transform.FindChild("Thumbnail").GetComponent<Image>().sprite = value));
                } else {
                    button.transform.FindChild("Thumbnail").GetComponent<Image>().sprite = noThumbnailSprite;
                }
#endif

                button.gameObject.SetActive(true);
            } else {
                break;
            }
            if (initializingGalleries) loadingBar.fillAmount = (((float)x / (float)GALLERY_ITEMS) / 2f);
            else loadingBar.fillAmount = ((float)x / (float)GALLERY_ITEMS);
            yield return null;
        }

        if (initializingGalleries) loadingBar.fillAmount = 1f / 2f;
        else {
            loadingBar.fillAmount = 1;
            loadingSprite.SetActive(false);
        }
        if (video_gallery_page == 0) prevPageButton.gameObject.SetActive(false);
        videoGalleryGrid.GetComponent<CanvasGroup>().alpha = 1;
    }

    // Carga de forma asincrona una pagina de la galeria de imagenes locales
    IEnumerator LoadImageGalleryPage() {
        imageGalleryGrid.GetComponent<CanvasGroup>().alpha = 0;
        if (!initializingGalleries) {
            loadingBar.fillAmount = 0;
            loadingSprite.SetActive(true);
        }

        foreach (Transform child in imageGalleryGrid.transform) {
            child.gameObject.SetActive(false);
            yield return null;
        }

        int i;
        int x;
        for (i = image_gallery_page * GALLERY_ITEMS, x = 0; x < GALLERY_ITEMS && i < images.Length; i++, x++) {
            if (imageGalleryGrid.transform.childCount > x) {
                var button = imageGalleryGrid.transform.GetChild(x).GetComponent<Button>();
                button.transform.FindChild("Path").GetComponent<Text>().text = images[i].FullName;
                button.transform.FindChild("Filename").GetComponent<Text>().text = images[i].Name;
                button.transform.SetParent(imageGalleryGrid.transform, false);

#if UNITY_ANDROID && !UNITY_EDITOR
                // Obtenemos el Thumbnail de Android
                string thumbPath = galleryJavaClass.CallStatic<string>("getImageThumbnailPath", unityAndroidJavaObject, images[i].FullName);
                if (Debug.isDebugBuild) Debug.Log("[THUMB] IMAGE: "+images[i].FullName+" THUMBNAIL: "+thumbPath);
                yield return null;

                // Pintamos Thumbnail en el Boton de la Galeria/grid    
                if (thumbPath != "") {
                    Texture2D texture = new Texture2D(2, 2);
                    yield return StartCoroutine(LoadThumbnailAsset("file://"+thumbPath, value => button.transform.FindChild("Thumbnail").GetComponent<Image>().sprite = value));
                } else {
                    button.transform.FindChild("Thumbnail").GetComponent<Image>().sprite = noThumbnailSprite;
                }
#endif

                button.gameObject.SetActive(true);
            } else {
                break;
            }
            loadingBar.fillAmount = (1f/2f)+(((float)x / (float)GALLERY_ITEMS) / 2f);
            yield return null;
        }

        loadingBar.fillAmount = 1;
        loadingSprite.SetActive(false);
        if (image_gallery_page == 0) prevPageButton.gameObject.SetActive(false);
        imageGalleryGrid.GetComponent<CanvasGroup>().alpha = 1;
        if (!imageGalleryAnimator.GetBool("isVisible") && !mainMenuAnimator.GetBool("isVisible")) {
            mainMenuAnimator.SetBool("isVisible", true);
            initializingGalleries = false;
        }

    }

    // Carga de forma asincrona una pagina de la galeria de videos 2D de Youtube
    IEnumerator LoadYoutube2DGalleryPage(YoutubeController.YoutubeResponse youtubeResponse, bool isFirstTime) {
        youtube2DGalleryGrid.GetComponent<CanvasGroup>().alpha = 0;
        loadingBar.fillAmount = 0;
        loadingSprite.SetActive(true);
        youtubeController.next2DPage = youtubeResponse.nextPageToken;
        youtubeController.prev2DPage = youtubeResponse.prevPageToken;

        if (youtubeGalleryAnimator.GetBool("isVisible")) {
            nextPageButton.gameObject.SetActive(youtubeController.next2DPage != null);
            prevPageButton.gameObject.SetActive(youtubeController.prev2DPage != null);
        }

        foreach (Transform child in youtube2DGalleryGrid.transform) {
            child.gameObject.SetActive(false);
            yield return null;
        }

        int i = 0;
        if (youtubeResponse != null) {
            foreach (YoutubeController.YoutubeResource youtubeResource in youtubeResponse.items) {

                // Añadimos el botón al grid de la galeria, indicando el path completo y el nombre del fichero
                if (youtube2DGalleryGrid.transform.childCount > i) {
                    var button = youtube2DGalleryGrid.transform.GetChild(i).GetComponent<Button>();
                    button.transform.FindChild("Path").GetComponent<Text>().text = youtubeResource.id.videoId;
                    button.transform.FindChild("Filename").GetComponent<Text>().text = youtubeResource.snippet.title;
                    button.transform.SetParent(youtube2DGalleryGrid.transform, false);
                    Texture2D texture = new Texture2D(2, 2);
                    yield return StartCoroutine(LoadThumbnailAsset(youtubeResource.snippet.thumbnails.medium.url, value => button.transform.FindChild("Thumbnail").GetComponent<Image>().sprite = value));
                    button.gameObject.SetActive(true);
                }

                i++;
                loadingBar.fillAmount = ((float)i / (float)GALLERY_ITEMS);
                yield return null;
            }
        }

        loadingBar.fillAmount = 1;
        loadingSprite.SetActive(false);
        youtube2DGalleryGrid.GetComponent<CanvasGroup>().alpha = 1;
        if (isFirstTime) {
            chooseYoutubeTypeAnimator.SetBool("isVisible", false);
            youtubeGalleryAnimator.SetBool("isVisible", true);
            nextPageButton.gameObject.SetActive(youtubeController.next2DPage != null);
            prevPageButton.gameObject.SetActive(youtubeController.prev2DPage != null);
        }
  
    }

    // Carga de forma asincrona una pagina de la galeria de videos 2D de 360
    IEnumerator LoadYoutube360GalleryPage(YoutubeController.YoutubeResponse youtubeResponse, bool isFirstTime) {
        youtube360GalleryGrid.GetComponent<CanvasGroup>().alpha = 0;
        loadingBar.fillAmount = 0;
        loadingSprite.SetActive(true);
        youtubeController.next360Page = youtubeResponse.nextPageToken;
        youtubeController.prev360Page = youtubeResponse.prevPageToken;

        if (youtubeGalleryAnimator.GetBool("isVisible")) {
            nextPageButton.gameObject.SetActive(youtubeController.next360Page != null);
            prevPageButton.gameObject.SetActive(youtubeController.prev360Page != null);
        }

        foreach (Transform child in youtube360GalleryGrid.transform) {
            child.gameObject.SetActive(false);
            yield return null;
        }

        int i = 0;
        if (youtubeResponse != null) {
            foreach (YoutubeController.YoutubeResource youtubeResource in youtubeResponse.items) {

                // Añadimos el botón al grid de la galeria, indicando el path completo y el nombre del fichero
                if (youtube360GalleryGrid.transform.childCount > i) {
                    var button = youtube360GalleryGrid.transform.GetChild(i).GetComponent<Button>();
                    button.transform.FindChild("Path").GetComponent<Text>().text = youtubeResource.id.videoId;
                    button.transform.FindChild("Filename").GetComponent<Text>().text = youtubeResource.snippet.title;
                    button.transform.SetParent(youtube360GalleryGrid.transform, false);
                    Texture2D texture = new Texture2D(2, 2);
                    yield return StartCoroutine(LoadThumbnailAsset(youtubeResource.snippet.thumbnails.medium.url, value => button.transform.FindChild("Thumbnail").GetComponent<Image>().sprite = value));
                    button.gameObject.SetActive(true);
                }

                i++;
                loadingBar.fillAmount = ((float)i / (float)GALLERY_ITEMS);
                yield return null;
            }
        }

        loadingBar.fillAmount = 1;
        loadingSprite.SetActive(false);
        youtube360GalleryGrid.GetComponent<CanvasGroup>().alpha = 1;
        if (isFirstTime) {
            chooseYoutubeTypeAnimator.SetBool("isVisible", false);
            youtube360GalleryAnimator.SetBool("isVisible", true);
            nextPageButton.gameObject.SetActive(youtubeController.next2DPage != null);
            prevPageButton.gameObject.SetActive(youtubeController.prev2DPage != null);

        }

    }

    // Devuelve, de forma asincrona, el Thumbnail de un video o imagen
    private IEnumerator LoadThumbnailAsset(string url, System.Action<Sprite> result) {
        WWW www = new WWW(url);
        float elapsedTime = 0.0f;

        yield return www;

        if (www.error == null) {
            Texture2D texTmp = new Texture2D(128, 128);
            www.LoadImageIntoTexture(texTmp);
            yield return null;

            Sprite sprTmp = Sprite.Create(texTmp, new Rect(0.0f, 0.0f, texTmp.width, texTmp.height), new Vector2(0.5f, 0.5f), 1000);

            result(sprTmp); // Pass retrieved result.
        } else {
            result(null);
        }
    }



}
