﻿/*
Autor: Vicente Manuel García Huete - vicman.lobo@gmail.com
Este software está licenciado como Creative Commons Atribución-NoComercial-SinDerivar 4.0 Internacional

Quedan excluídos de la Licencia, y se prohíbe su redistribución, los siguientes recursos con Licencias Privadas:
Directorio: "Assets/EasyMovieTexture" (https://www.assetstore.unity3d.com/en/#!/content/10032)
Directorio: "Assets/Coindrop assets" (https://www.assetstore.unity3d.com/en/#!/content/28039)

Recursos con Licencias Creative Commons Utilizados:
YoutubeExtractor: https://github.com/flagbug/YoutubeExtractor
Youtube Data API v3: https://developers.google.com/youtube/v3/
Wallpaper Tablet: https://www.flickr.com/photos/101039078@N06/9759631834
Sprite de Carga: https://pixabay.com/es/proyector-proyector-cinematogr%C3%A1fico-361784/
Icono Youtube: https://pixabay.com/es/video-jugar-multimedia-youtube-481821/
Icono Imagen: https://commons.wikimedia.org/wiki/File:Image_manquante_2.svg
Iconos Zoom: https://commons.wikimedia.org/wiki/File:Zoom_out_font_awesome.svg
https://commons.wikimedia.org/wiki/File:Zoom_in_font_awesome.svg
Icono Salir: https://pixabay.com/es/registro-fuera-cerrar-sesi%C3%B3n-salida-35496/
Icono Mute: https://commons.wikimedia.org/wiki/File:Speaker_Icon.svg#/media/File:Mute_Icon.svg
Icono Volume Up: https://commons.wikimedia.org/wiki/File:Speaker_Icon.svg
Iconos Reproducción: https://pixabay.com/es/multimedia-botones-jugador-jugar-153230/
Icono Sprite Gaze: https://commons.wikimedia.org/wiki/File:Map_pointer.svg
Modelo Tablet: http://www.modelplusmodel.com/tech/apple-ipad-2.html
Icono Video: https://pixabay.com/es/cuenta-regresiva-cine-pel%C3%ADcula-155439/
Icono Preferencias: https://pixabay.com/es/engranajes-opciones-configuraci%C3%B3n-467261/
Icono Galería: https://pixabay.com/es/portapapeles-pel%C3%ADcula-clap-305778/
Fuente Cinerama: http://www.dafont.com/es/cinerama.font
*/

using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using YoutubeExtractor;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using Newtonsoft.Json;
using System.ComponentModel;

public class YoutubeController : MonoBehaviour {
    public class YoutubeResource {
        public string kind;                         // Tipo de recurso de la API
        public string etag;                         // Etag de este recurso.
        public class ResourceId {                   
            public string kind;                     // Tipo de recurso de la API
            public string videoId;                  // Si el valor de la propiedad id.type es youtube#video, esta propiedad estará presente y su valor contendrá el ID que YouTube utiliza para identificar de forma exclusiva un video que coincide con la consulta de búsqueda
            public string channelId;                // Si el valor de la propiedad id.type es youtube#channel, esta propiedad estará presente y su valor contendrá el ID que YouTube utiliza para identificar de forma exclusiva un canal que coincide con la consulta de búsqueda
            public string playlistId;               // Si el valor de la propiedad id.type es youtube#playlist, esta propiedad estará presente y su valor contendrá el ID que YouTube utiliza para identificar de forma exclusiva una lista de reproducción que coincide con la consulta de búsqueda
        }
        public ResourceId id;                       // El objeto id contiene información que se puede usar para i­dentificar de forma exclusiva el recurso que coinci­de con la solicitud de búsqueda.
        public class Snippet {
            public DateTime publishedAt;            // Fecha y hora de creación del recurso que identifica el resultado de búsqueda. El valor se especifica en el formato ISO 8601 (YYYY-MM-DDThh:mm:ss.sZ)
            public string channelId;                // Valor que YouTube utiliza para identificar de forma exclusiva el canal que publicó el recurso que el resultado de búsqueda identifica
            public string title;                    

            public string description;
            public class Thumbnails {
                public class Thumbnail {
                    public string url;
                    public int width;
                    public int height;
                }
                public Thumbnail @default; // default es palabra reservada
                public Thumbnail medium;
                public Thumbnail high;
            }
            public Thumbnails thumbnails;
            public string channelTitle;
        }
        public Snippet snippet;                     // El objeto snippet contiene información básica sobre un resultado de búsqueda
    }

    public class YoutubeResponse {
        public string kind;
        public string etag;
        public string nextPageToken;
        public string prevPageToken;
        public class PageInfo {
            public int totalResults;
            public int resultsPerPage;
        }
        public PageInfo pageInfo;
        public YoutubeResource[] items;
    }

    public class YoutubeRequest {
        // Obligatorios
        public string part;                         // "id","snippet"

        // Filtros
        public bool forContentOwner=false;          // Restringe la búsqueda para recuperar solo los recursos del propietario del contenido, especificado por el parámetro onBehalfOfContentOwner 
        public bool forMine=false;                  // Restringe la búsqueda para recuperar solo los videos que pertenecen al usuario autenticado.
        public string relatedToVideoId;             // Recupera una lista de videos relacionados con el video que identifica el valor del parámetro

        // Opcionales
        public string channelId;                    // Indica que la respuesta de la API debe contener solamente recursos creados por el canal.
        public string channelType;                  // "any", "show"
        public string eventType;                    // "completed", "live", "upcoming"
        public int maxResults=5;                      // Resultados por pagina: 0 a 50 ambos inclusive (por defecto es 5)
        public string onBehalfOfContentOwner;       // Indica que las credenciales de autorización de la solicitud identifican a un usuario de CMS de YouTube que actúa en nombre del propietario de contenido especificado en el valor del parámetro
        public string order;                        // "date", "rating", "relevance", "title", "videoCount", "viewCount"
        public string pageToken;                    // Identifica una página específica en el conjunto de resultados que se debe mostrar. En una respuesta de la API, las propiedades nextPageToken y prevPageToken identifican otras páginas que se podrían recuperar 
        public DateTime publishedAfter;             // Formato RFC 3339 (1970-01-01T00:00:00Z) contener solamente recursos creados después de la hora especificada
        public DateTime publishedBefore;            // Formato RFC 3339 (1970-01-01T00:00:00Z) ontener solamente recursos creados antes de la hora especificada
        public string q;                            // Especifica el término de consulta que se debe buscar
        public string regionCode;                   // Indica a la API que muestre los resultados de búsqueda para el país especificado. El valor del parámetro es un código de país ISO 3166-1 alpha-2.
        public string safeSearch;                   // "moderate", "none", "strict" Indica si los resultados de la búsqueda deben incluir contenido restringido además de contenido estándar.
        public string topicId;                      // Indica que la respuesta de la API solo debe contener recursos asociados con el tema especificado
        public string type;                         // "channel", "playlist", "video" Restringe una consulta de búsqueda para recuperar solo un tipo de recurso en particular
        public string videoCaption;                 // "any","closedCatp
        public string videoCategoryId;              // Filtra los resultados de búsqueda de videos según su categoría
        public string videoDefinition;              // "any", "high", "standard" Permite restringir una búsqueda para incluir solo los videos de alta definición (HD) o de definición estándar (SD).
        public string videoDimension;               // "2d", "3d", "any" Permite restringir la búsqueda para recuperar solo videos en 2D o 3D.
        public string videoDuration;                // "any", "long", "medium", "short" Filtra los resultados de búsqueda de videos según su duración: cualquiera, 20m, 4-20m, 4m
        public string videoEmbeddable;              // "any", "true" Restringir una búsqueda para incluir solo videos que se puedan insertar en una página web.
        public string videoLicense;                 // "any", "creativeCommon", "youtube" Filtra los resultados de búsqueda para incluir solo los videos con una licencia en particular
        public string videoSyndicated;              // "any", "true" Restringir una búsqueda para incluir solo los videos que se puedan reproducir fuera de youtube.com.
        public string videoType;                    // "any", "episode", "movie" restringir una búsqueda para incluir un determinado tipo de videos: todos, programas, peliculas
        public string GetUriParams() {              // Obtiene la cadena de parametros de la URL
            string uriParams = "?";
            uriParams += (part != null) ? "part=" + part : "part=snippet";
            if (forContentOwner == true) uriParams += "&forContentOwner=" + forContentOwner;
            if (forMine == true) uriParams += "&forMine=" + forMine;
            if (relatedToVideoId != null) uriParams += "&relatedToVideoId=" + relatedToVideoId;
            if (channelId != null) uriParams += "&channelId=" + channelId;
            if (channelType != null) uriParams += "&channelType=" + channelType;
            if (eventType != null) uriParams += "&eventType=" + eventType;
            if (maxResults != 5) uriParams += "&maxResults=" + maxResults;
            if (onBehalfOfContentOwner != null) uriParams += "&onBehalfOfContentOwner=" + onBehalfOfContentOwner;
            if (order != null) uriParams += "&order=" + order;
            if (pageToken != null) uriParams += "&pageToken=" + pageToken;
            if (publishedAfter != DateTime.MinValue) uriParams += "&publishedAfter=" + publishedAfter;
            if (publishedBefore != DateTime.MinValue) uriParams += "&publishedBefore=" + publishedBefore;
            if (q != null) uriParams += "&q=" + q;
            if (regionCode != null) uriParams += "&regionCode=" + regionCode;
            if (safeSearch != null) uriParams += "&safeSearch=" + safeSearch;
            if (topicId != null) uriParams += "&topicId=" + topicId;
            if (type != null) uriParams += "&type=" + type;
            if (videoCaption != null) uriParams += "&videoCaption=" + videoCaption;
            if (videoCategoryId != null) uriParams += "&videoCategoryId=" + videoCategoryId;
            if (videoDefinition != null) uriParams += "&videoDefinition=" + videoDefinition;
            if (videoDimension != null) uriParams += "&videoDimension=" + videoDimension;
            if (videoDuration != null) uriParams += "&videoDuration=" + videoDuration;
            if (videoEmbeddable != null) uriParams += "&videoEmbeddable=" + videoEmbeddable;
            if (videoLicense != null) uriParams += "&videoLicense=" + videoLicense;
            if (videoSyndicated != null) uriParams += "&videoSyndicated=" + videoSyndicated;
            if (videoType != null) uriParams += "&videoType=" + videoType;
            return uriParams;
        }
    }

    // Sobreescribe la politica de seguridad de mono para aceptar el certificado de Google
    public class UnsafeSecurityPolicy {
        public static bool Validator(
            object sender,
            X509Certificate certificate,
            X509Chain chain,
            SslPolicyErrors policyErrors) {

            //*** Just accept and move on...
            Debug.Log("Validation successful!");
            return true;
        }

        public static void Instate() {
            ServicePointManager.ServerCertificateValidationCallback = Validator;
        }
    }

    public const string YOUTUBE_URL = "https://www.googleapis.com/youtube/v3/search";
    private const string ANDROID_API_KEY = "AIzaSyCKSqaWUwSbXrI0jyDVnLr7w9JRXgQLzeo";   //https://console.developers.google.com/apis/library?project=vv360-1154
    private const string EDITOR_API_KEY = "AIzaSyAJqXIhvLIuAnZsdz_0z2e5n4ZfO1Mq7zE";
    public const string YOUTUBE_VIDEO_URL = "http://www.youtube.com/watch?v=";

    public MediaPlayerCtrl mediaPlayer;
    public string next2DPage;
    public string prev2DPage;
    public string next360Page;
    public string prev360Page;

    void Start() {
        UnsafeSecurityPolicy.Instate();
    }

    public IEnumerator MediaScan(string pageToken, bool is360, System.Action<YoutubeResponse> responseCallBack) {
        YoutubeRequest youtubeRequest = new YoutubeRequest();
        youtubeRequest.part = "snippet";
        youtubeRequest.type = "video";
        if (is360) {
            youtubeRequest.q = "360video";
        } else {
            youtubeRequest.channelId = "UCm3FGJTmlb30B71183Pza4A"; // ID del Canal UOC
        }

        youtubeRequest.maxResults = 8;
        if (pageToken != "") youtubeRequest.pageToken = pageToken;        

        Debug.Log("[YOUTUBE] GET: " +YOUTUBE_URL + youtubeRequest.GetUriParams()+ "&key="+ANDROID_API_KEY);
        YoutubeResponse youtubeResponse = new YoutubeResponse();
        WWW www = new WWW(YOUTUBE_URL + youtubeRequest.GetUriParams()+"&key="+ANDROID_API_KEY);
        yield return StartCoroutine(WaitForRequest(www, (value) => youtubeResponse = value));
        Debug.Log("[YOUTUBE] RESPONSE: "+JsonConvert.SerializeObject(youtubeResponse));
        if (youtubeResponse != null) responseCallBack(youtubeResponse);
        else responseCallBack(null);
    }

    IEnumerator WaitForRequest(WWW www, System.Action<YoutubeResponse> responseCallBack) {
        yield return www;
        if (www.error == null) {
            //Movie m = JsonConvert.DeserializeObject<Movie>(json)
            responseCallBack(JsonConvert.DeserializeObject<YoutubeResponse>(www.text));
        } else {
            responseCallBack(null);
            Debug.Log("[YOUTUBE] " + www.error);
        }
    }

    private string RemoveIllegalPathCharacters(string path) {
        string regexSearch = new string(Path.GetInvalidFileNameChars()) + new string(Path.GetInvalidPathChars());
        var r = new Regex(string.Format("[{0}]", Regex.Escape(regexSearch)));
        return r.Replace(path, "");
    }
}
